-- ------ --
-- Damage --
-- ------ --

local damage = class('damage')

-- elements table contains info on each element and what it is strong/weak against
local elements       = {}
  elements[1]        = "plant"
  elements[2]        = "earth"
  elements[3]        = "water"
  elements[4]        = "fire"
  elements[5]        = "metal"
  elements["plant"]  = { plant=false, earth=true,  water=true,  fire=false, metal=false }
  elements["earth"]  = { plant=false, earth=false, water=true,  fire=true,  metal=false }
  elements["water"]  = { plant=false, earth=false, water=false, fire=true,  metal=true  }
  elements["fire"]   = { plant=true,  earth=false, water=false, fire=false, metal=true  }
  elements["metal"]  = { plant=true,  earth=true,  water=false, fire=false, metal=false }

local killIndex = {}
local hitIndex = {}

-- kill type table contains info on all attack types used by the player
-- b = bounce value
-- s = hitbox shape
-- d = base damage
-- p = affected by power up
-- c = collision/pushback
killType = {}
  killType["slash"]  = { b=false, s="wide",   d=10, p=true,  c=false }
  killType["vslash"] = { b=false, s="high",   d=10, p=true,  c=false }
  killType["dive"]   = { b="y",   s="square", d= 5, p=false, c=false }
  killType["slide"]  = { b=false, s=false,    d= 5, p=false, c=false }
  killType["tackle"] = { b="v",   s="square", d=25, p=false, c=false }
  killType["upper"]  = { b=false, s="high",   d=25, p=false, c=false }
  killType["splash"] = { b=false, s="splash", d=25, p=false, c=true  }

-- global damage table contains numeric modifiers for various situations
globalDamage = {}
  globalDamage["lava"] = {b=10,l=1}

function damageMult(l)
  if l < 4 then l = 4 end
  local mult = math.floor(math.sqrt(l-4))+1
  -- local mult = math.max((l-1)^2,1)+4
  return mult
end

function checkElement(e,t)
  return elements[e][t]
end

function getElement(n)
  return elements[n]
end

function initKill(p,h,k)
  hitIndex[p] = h
  killIndex[p] = k
end

function doHit(p)
  hitIndex[p] = tonumber(hitIndex[p]) + 1
end

function doKill(p)
  killIndex[p] = tonumber(killIndex[p]) + 1
end

function getKillType(k,t)
  if t then
    return killType[k][t]
  else
    return killType[k]
  end
end

function getKillBounce(k)
  return killType[k]["bounce"]
end

function getKillShape(k)
  return killType[k]["shape"]
end

function getKill(p)
  return killIndex[p]
end

function getHit(p)
  return hitIndex[p]
end

function countKill()
  return #killIndex
end

function countHit()
  return #hitIndex
end

function updateDamage()
  for k,v in pairs(damageObjects) do
    if (os.time() - v[4] < 2) then
      v[3] = v[3] - 1   --  this may break on windows without vsync
    else
      k = nil
    end
  end
end

function resetDamage()
  damageObjects = {}
end

function displayDamage(e,d)             -- entity, damage
  daml,damt = (e.l + e.w/2 - 8),(e.t - 48)
  dami = os.time()
  damr = tostring(math.random() .. dami)
  damo = {d,daml,damt,dami}
  damageObjects[damr] = damo
end

resetDamage()

return damage
