-- this file has common behaviour for entities affected by gravity (The player and monsters)

local bump          = require 'lib.bump'
local Entity        = require 'entities.Entity'

local GravityEntity = class('GravityEntity', Entity)

local gravityAccel  = 1600              -- gravity, in pixels per second per second
local minVelocity   =   10

local abs = math.abs

local function pad(x, min, max)
  return x < min and min or (x > max and max or x)
end

local function zero(x, limit)
  return abs(x) < limit and 0 or x
end

local function padVelocity(maxdt, vx, vy)
  local max = bump.getCellSize()/maxdt*0.5
  local min = -max
  return pad(vx, min, max), pad(vy, min, max)
end

function GravityEntity:initialize(x,y,w,h)
  Entity.initialize(self, x,y,w,h)
  self.vx, self.vy = 0,0
end

function GravityEntity:update(dt, maxdt, fake)
  if fake then
    local vy = self.vy + gravityAccel * dt
    local vx,vy = padVelocity(maxdt, self.vx, self.vy)
    local l,t = self.l + self.vx * dt, self.t + self.vy * dt
    return l,t,vx,vy,gravityAccel
  else
  self.vy = self.vy + gravityAccel * dt
  self.vx, self.vy = padVelocity(maxdt, self.vx, self.vy)
  self.l, self.t = self.l + self.vx * dt, self.t + self.vy * dt
  end
  local ut = mapHeight
  if self.t > ut then self.t = self.t - ut end
  if self.t < 0 then self.t = self.t + ut end
end

return GravityEntity
