-- represents the hitbox of a particular move

local Entity  = require 'entities.Entity'
local Monster = require 'entities.Monster'
local damage  = require 'data.damage'

local Hitbox = class('Hitbox', Entity)

local hitboxes = {} -- this will hold a list of all active hitboxes

function Hitbox:initialize(l,t,w,h,p,e,d,c) -- p player, e element, d damage, c collision
  self.huid = tostring(math.random())
  Entity.initialize(self, l,t,w,h)
  if e then
    self.element = e
  else
    self.element = false
  end
  if d then
    self.damage = d
  else
    self.damage = 1
  end
  if c then
    self.push = true
  else
    self.push = false
  end
  hitboxes[p] = self
end

function Hitbox:shouldCollide(other)
  if self.element and other.element and (self.element == other.element) then
    return false                        --  No damage if the target is of the same element
  else
    if (other.hitby == self.huid) then
      return false                      --  Avoid multiple hits by the same hitbox
    else
      return instanceOf(Monster, other)
    end
  end
end

function Hitbox:collision(other, dx, dy)
  p = self:resolve(self)                --  find the player
  local mult = damageMult(p.score["level"])
  doHit(p)                              --  increment player hit counter
  p.score["combo"] = p.score["combo"] + 1
  if self.element then                  --  pass aligned element if it exists
    other:takedamage((self.damage + p.score["level"])*mult,self.huid,p,self.element)
  else                                  --  damage formula
    other:takedamage((self.damage + p.score["level"])*mult,self.huid,p)
  end
end

function Hitbox:move(l,t)
  self.l,self.t = l,t
end

function Hitbox:retrieve(p) -- given a player, return associated hitbox
  return hitboxes[p]
end

function Hitbox:resolve(h) -- given a hitbox, return associated player
  for k,v in pairs(hitboxes) do
    if v == h then return k end
  end
end

function Hitbox:remove(p)
  hitboxes[p] = nil
end

function Hitbox:draw()
  -- love.graphics.setColor(255,0,0,100) -- we're not going to fill these in
  -- love.graphics.rectangle('fill', self:getBBox())
  love.graphics.setColor(255,0,0)
  love.graphics.rectangle('line', self:getBBox())
end

return Hitbox
