-- represents the Lockon ability

local Entity = require 'entities.Entity'
local Block  = require 'entities.Block'
local Hitbox = require 'entities.Hitbox'
local damage = require 'data.damage'

local Lockon   = class('Lockon', Entity)

local lockons  = {} -- this will hold a list of all active hitboxes

local abs = math.abs

local function sign(x)
  return x < 0 and -1 or (x > 0 and 1 or 0)
end

function Lockon:initialize(l,t,w,h,p,d) -- designed to be called with a player argument
  Entity.initialize(self, l,t,w,h)
  self.hit = false
  self.target = false
  if d then self.direction = d end
  lockons[p] = self
  -- p.distance = p:getSwing()
end

function Lockon:shouldCollide(other)
  -- if other.type~="slick" then
    return instanceOf(Block, other)
  -- end
end

function Lockon:gethit(p)
  return self.hit
end

function Lockon:collision(other, dx, dy)
  self.hit = true
  self.target = other
  -- other:change()
  p = self:resolve(self)
  p.distance = p:getSwing()
  if other.type ~= "slick" then
    p.score["combo"] = p.score["combo"] + 1
  end
  -- p.vy = 640
  -- p.vy = 0
  -- p.vx = sign(p.vx) * (math.abs(p.vx) + (320 - math.abs(p.vx)))
  self.vx = 0
  self.vy = 0
  -- self.l, self.t = self.l + dx, self.t + dy
  self.l, self.t = self.l + dx, self.t + dy
  -- self.l, self.t = self.l, self.t 
  -- some code for hitting the block
end

function Lockon:move(l,t)
  self.l,self.t = l,t
end

function Lockon:retrieve(p) -- given a player, return associated lock
  return lockons[p]
end

function Lockon:resolve(l) -- given a lock, return associated player
  for k,v in pairs(lockons) do
    if v == l then return k end
  end
end

function Lockon:remove(p)
  lockons[p] = nil
end

function Lockon:update(p,dt,maxdt)
  -- local lockspeed = 128
  local lockspeed = 160
  local l,t = self.l, self.t
  if not self.hit then                  --  we haven't collided yet, move the lockon
    if self.direction then
      if (self.direction == 8) then
        t = t - lockspeed * dt
      elseif (self.direction == 9) then
        t = t - lockspeed/(math.sqrt(200)/10) * dt
        l = l + lockspeed/(math.sqrt(200)/10) * dt
      elseif (self.direction == 7) then
        t = t - lockspeed/(math.sqrt(200)/10) * dt
        l = l - lockspeed/(math.sqrt(200)/10) * dt
      elseif (self.direction == 2) then
        t = t + lockspeed * dt
      elseif (self.direction == 4) then
        l = l - lockspeed * dt
      elseif (self.direction == 6) then
        l = l + lockspeed * dt
      end
    else
      t = t - lockspeed * dt
    end
  else                                  --  we hit something, check what type it is
    
  end
  local ut = mapHeight
  if t > ut then t = t - ut end
  if t < 0 then t = t + ut end
  -- l = p.l + 8
  -- if t > 
  self.l,self.t = l,t
end

function Lockon:draw()
  love.graphics.setColor(255,255,255,100)
  love.graphics.rectangle('fill', self:getBBox())
  love.graphics.setColor(255,255,255)
  local p = self:resolve(self)
  local px,py = p:getCenter()
  --  love.graphics.line(px,py, self:getCenter())
  love.graphics.rectangle('line', self:getBBox())
end

return Lockon
