-- -------- --
-- Monsters --
-- -------- --

local GravityEntity = require 'entities.GravityEntity'
local Block         = require 'entities.Block'
local Object        = require 'entities.Object'
local damage        = require 'data.damage'
local cron          = require 'lib.cron'

local Monster       = class('Monster', GravityEntity)

local bounceRate = 1
local jumpVelocity = 800
local abs = math.abs

local function sign(x)
  return x < 0 and -1 or (x > 0 and 1 or 0)
end

local E = {1,2,3,4,5}
for k,v in ipairs(E) do
  E[k] = getElement(k)
end

function Monster:initialize(x,y,vx,vy,l,b)
  randE = math.random(0,5)
  self.element = getElement(randE)      --  assign a random element
  self.damage = math.random(5,25)       --  how much damage we do per hit
  self.life = math.random(10,50)        --  how much life we have
  self.hitby = false                    --  remembers a hitbox that hit us
  self.damagetaken = false              --  tracks getting hit
  self.move = false                     --  movement patterns
  self.jump = false                     --  jump bit
  self.push = false                     --  push bit
  if b then 
    self.boss = true                    --  boss bit
  else
    self.boss = false
  end
  self.size = (math.floor( (10-math.floor(self.damage/5)) / 2) * 8) + 16
  if l then
    self.damage = math.floor(self.damage * math.random(1,l))
    self.life = math.floor(self.life * math.random(1,l))
  end
  GravityEntity.initialize(self,x,y, self.size, self.size)
  self.vx, self.vy = vx or 0, vy or 0
end

function Monster:change()
  randE = math.random(0,5)
  self.element = getElement(randE)      --  assign a random element
end

function Monster:shouldCollide(other)
  -- if other.push then
  --   return instanceOf(Block, other) or instanceOf(Monster, other) or instanceOf(Hitbox, other)
  -- else
    return instanceOf(Block, other) or instanceOf(Monster, other)
  -- end
end

function Monster:collision(other, dx, dy)
  -- if not other.huid or (other.huid and other.push) then 
    if not other.static and self.boss == true then
      other:change()
    end
      local bounceAdj = bounceRate
    if self.boss == true then  -- bosses go faster
      -- bounceAdj = bounceAdj * 2 -- too much
    end
    if dx~=0 then
      self.l = self.l + dx
      self.vx = abs(self.vx) * sign(dx) * (bounceAdj)
    end
    if dy~=0 then
      self.t = self.t + dy
      self.vy = abs(self.vy) * sign(dy) * (bounceAdj + math.random())
    end
    if other.push and other.huid then
      self.push = true
      cron.tagged('push').cancel()
      cron.tagged('push').after(0.6, function() self.push = false end)
    end
  -- end
end

function Monster:takedamage(d,h,p,e)
  self.hitby = h
  self.damagetaken = d
  if e then
    if (self.element and checkElement(e,self.element)) then
      self.damagetaken = self.damagetaken * 2
    end
  end
  cron.tagged('monsterhit').cancel()
  cron.tagged('monsterhit').after(0.4, function() self.damagetaken = false end)
  self.life = math.max((self.life - self.damagetaken), 0)
  if (self.life == 0) then
    doKill(p)
  end
  if not self.boss then
  local sdx,sdy = self:getCenter()
  local pdx,pdy = p:getCenter()
  if sdx < pdx then
    self.vx = abs(self.vx) * -1
  elseif sdx > pdx then
    self.vx = abs(self.vx)
  end
  end
  displayDamage(self,self.damagetaken) 
end

function Monster:dropObject()
  local droprate = math.random(1,100)
  local droptype = "sashimi"
  -- if (droprate >= 83) then
  if (droprate >= 1) then -- do this so every monster drops sushi
    if self.element == "plant" then
      droptype = "nori"
    elseif self.element == "earth" then
      droptype = "enoki"
    elseif self.element == "water" then
      droptype = "miso"
    elseif self.element == "fire" then
      droptype = "wasabi"
    elseif self.element == "metal" then
      droptype = "tekka"
    end
    Object:new(self.l,self.t,droptype)
  end
  if self.boss == true then
    Object:new(self.l,self.t,"maptoken")
  end
end

function Monster:update(dt, maxdt)
  if (self.life <= 0) then
    self:dropObject()
    self:destroy()
  end
  randX = math.random(1,1000)
  if (randX < 3) then                   --  occasional direction switches
    if not self.push then
      self.vx = -(self.vx)
    end
  end
  if self.boss then
    self.vx = sign(self.vx) * (2*jumpVelocity/5)  --  bosses move faster
    bossx,bossy = self:getCenter()
  else
    if self.push then
      self.vx = sign(self.vx) * (2*jumpVelocity/5)
    else
      self.vx = sign(self.vx) * (jumpVelocity/5)
    end
  end
  if self.move then                     --  a move pattern is set
    if self.move == 1 then              --  pattern 1, normal
      GravityEntity.update(self, dt, maxdt)
    elseif self.move == 2 then          --  pattern 2, jump
      if not self.jump then
        self.vy = -jumpVelocity
        self.jump = true
        cron.tagged('jump').after(0.6, function() self.jump = false end)
      end
      GravityEntity.update(self, dt, maxdt)
    else                                --  pattern 3, hang in midair
      GravityEntity.update(self, dt, maxdt)
      self.vy = 0
    end
    -- cron.tagged('behavior').cancel()
  else                                  --  a move pattern is not set, set one
    self.jump = false                   --  reset jump bit
    local behaviorlength = math.random(2,4)
    local behaviorchange = math.random(1,3)
    self.move = behaviorchange
    cron.tagged('behavior').after(behaviorlength, function() self.move = false end)
    GravityEntity.update(self, dt, maxdt)
  end
end

function Monster:draw()
  local l,t,w,h = self:getBBox()
  local e = false
  if self.element then
    e = string.upper(string.sub(self.element,1,1))
  end
  -- do this to display move state instead of element
  -- if self.move then e = string.upper(string.sub(self.move,1,1)) else e = 0 end
  love.graphics.setColor(220,150,150,100)
  love.graphics.rectangle('fill', l,t,w,h)
  love.graphics.setColor(220,150,150)
  love.graphics.rectangle('line', l,t,w,h)
  love.graphics.setColor(255,255,255)
  if e then
    love.graphics.setColor(255,255,255)
    love.graphics.setFont(bigFont)
    love.graphics.print(e,l,t)
    love.graphics.setFont(defaultFont)
  end
  if self.boss == true then
    love.graphics.print("boss",l,t-(self.h/2))
  end
  if self.push == true then
    love.graphics.print("push",l,t-(self.h/2))
  end
end

return Monster
