-- ------- --
-- Objects --
-- ------- --

local GravityEntity                     = require 'entities.GravityEntity'
local Block                             = require 'entities.Block'

local Inventory                         = require 'data.inventory'

local Object                            = class('Object', GravityEntity)

local inertia                           = 120
local minVelocity                       =  10
local bounceRate = 0.1
local abs = math.abs

local function sign(x)
  return x < 0 and -1 or (x > 0 and 1 or 0)
end

function Object:initialize(x,y,t)
  if t and objectTypes[t] then          --  Valid object type specified
    self.n = objectTypes[t]
    self.type = t
  else                                  --  Use the base type
    self.type = "base"
    self.n = objectTypes["base"]
  end
  GravityEntity.initialize(self,x,y,16,16)
  self.vx = math.random(-160,160)
end

function Object:shouldCollide(other)
  return instanceOf(Block, other)
end

function Object:collision(other, dx, dy)
  --  Gravity is currently too strong to allow these to bounce.
  if dx~=0 then
    self.l = self.l + dx
    self.vx = abs(self.vx) * sign(dx) * bounceRate
  end
  if dy~=0 then
    self.t = self.t + dy
    self.vy = abs(self.vy) * sign(dy) * bounceRate
  end
end

function Object:update(dt, maxdt)
  vx = self.vx
  vx = vx - dt * inertia * sign(vx)
  if abs(vx) < minVelocity then vx = 0 end
  GravityEntity.update(self,dt,maxdt)
  self.vx = vx
end

function Object:draw()
  love.graphics.setColor(255,0,255,100)
  love.graphics.rectangle('fill', self:getBBox())
  love.graphics.setColor(255,255,255)
  love.graphics.rectangle('line', self:getBBox())
  -- if self.n.t then
  --   love.graphics.print(string.upper(string.sub(self.n.t,1,1)),self.l,self.t-self.h)
  -- else
    love.graphics.print(self.type,self.l,self.t-self.h)
  -- end
end

return Object
